use strict;
use warnings;
use Test::More tests => 4;

use_ok 'EV::Kafka';

my $conn = EV::Kafka::Conn::_new('EV::Kafka::Conn', undef);
isa_ok $conn, 'EV::Kafka::Conn';
ok !$conn->connected, 'not connected initially';

my $client = EV::Kafka->new(brokers => '127.0.0.1:9092');
isa_ok $client, 'EV::Kafka::Client';
